package com.bergerkiller.bukkit.nolagg.chunks;

import java.util.logging.Level;

import org.bukkit.Bukkit;
import org.bukkit.permissions.PermissionDefault;
import org.bukkit.plugin.Plugin;

import com.bergerkiller.bukkit.common.config.FileConfiguration;
import com.bergerkiller.bukkit.nolagg.base.NoLaggBase;

public class NoLaggChunks extends NoLaggBase {

	public static boolean isOreObfEnabled = false;
	public static boolean isRawCritOrbEnabled = false;
	public static boolean isSpoutEnabled = false;
	public static boolean useBufferedLoading = true;

	@Override
	public void updateDependency(Plugin plugin, String pluginName, boolean enabled) {
		if (pluginName.equals("Spout")) {
			isSpoutEnabled = enabled;
		} else if (pluginName.equals("Chunk Manager")) {
			if (enabled) {
				log(Level.WARNING, "Chunk Manager detected, NoLaggChunks has been disabled.");
				log(Level.WARNING, "Either disable Chunk Manager or disable NoLaggChunks.");
				Bukkit.getServer().getPluginManager().disablePlugin(this);
			}
		} else {
			if (pluginName.equals("Orebfuscator")) {
				if (isOreObfEnabled = enabled && useBufferedLoading) {
					log(Level.INFO, "Orebfuscation has been detected and will be used when sending chunks");
					log(Level.INFO, "Note that this may require you to set more threads used for sending!");
				}
			} else if (pluginName.equals("RawcriticsOreObfuscationPluginSpout")) {
				if (isRawCritOrbEnabled = enabled && useBufferedLoading) {
					//log(Level.INFO, "Raw Critics Ore Obfuscation has been detected and will be used when sending chunks");
					//log(Level.INFO, "Note that this may require you to set more threads used for sending!");
				}
			}
		}
	}

	public static NoLaggChunks plugin;

	@Override
	public void reload() {
		FileConfiguration config = new FileConfiguration(this);
		config.load();

		config.setHeader("This is the configuration of NoLaggChunks\n");
		config.setHeader("minRate", "The minimum chunk sending rate (chunks/tick)");
		config.setHeader("maxRate", "The maximum chunk sending rate (chunks/tick)");
		
		config.setHeader("bufferedLoader", "If you use a plugin that depends on the net server handler for packets, disable this");
		config.addHeader("bufferedLoader", "For example, Raw Critics' Ore Obfuscation does not function with this enabled.");
		config.addHeader("bufferedLoader", "Orebfuscator is supported and works with this enabled.");

		config.setHeader("triggerRate", "The rate at which chunks are 'triggered' (refreshed, make visible) (trigger packets/tick)");
		config.setHeader("bufferedLoader.enabled", "Whether or not to use the buffered packet loader to reduce new memory allocation");
		config.setHeader("bufferedLoader.threadCount", "The amount of threads to use to compress the chunk packets (increase if it can't keep up)");
		ChunkSendQueue.minRate = config.get("minRate", 0.25);
		ChunkSendQueue.maxRate = config.get("maxRate", 2.00);
		ChunkSendQueue.globalTriggerRate = config.get("triggerRate", 0.5);
		useBufferedLoading = config.get("bufferedLoader.enabled", true);
		ChunkCompressionThread.init(config.get("bufferedLoader.threadCount", 2));

		config.save();
	}

	public void enable() {
		plugin = this;

		this.register(NLCListener.class);
		this.register(new Commands());

		this.reload();

		ChunkSendQueue.init();
	}

	public void disable() {
		ChunkSendQueue.deinit();
		ChunkCompressionThread.deinit();
	}

	@Override
	public void permissions() {
		this.loadPermission("nolagg.chunks.sending", PermissionDefault.OP, "The player can view sending information about himself");
	}

}
